@@ -153,6 +153,13 @@ ExchangeMapiConnection *exchange_mapi_connection_find (const gchar *profile);
 gboolean		exchange_mapi_connection_reconnect (ExchangeMapiConnection *conn, const gchar *password);
 gboolean		exchange_mapi_connection_close (ExchangeMapiConnection *conn);
 gboolean		exchange_mapi_connection_connected (ExchangeMapiConnection *conn);
+
+gboolean		exchange_mapi_connection_fetch_object_props (
+					ExchangeMapiConnection *conn, mapi_object_t *obj_folder, mapi_id_t fid, mapi_id_t mid, mapi_object_t *obj_message,
+					BuildReadPropsCB build_props, gpointer brp_data,
+					FetchCallback cb, gpointer data,
+					guint32 options);
+
 gboolean		exchange_mapi_connection_fetch_item (ExchangeMapiConnection *conn, mapi_id_t fid, mapi_id_t mid,
 					BuildReadPropsCB build_props, gpointer brp_data,
 					FetchCallback cb, gpointer data,
@@ -168,6 +175,8 @@ gboolean		exchange_mapi_connection_fetch_gal (ExchangeMapiConnection *conn,
 					BuildReadPropsCB build_props, gpointer brp_data,
 					FetchGALCallback cb, gpointer data);
 
+gboolean		exchange_mapi_connection_get_public_folder (ExchangeMapiConnection *conn, mapi_object_t *obj_object);
+
 mapi_id_t		exchange_mapi_connection_create_folder (ExchangeMapiConnection *conn, uint32_t olFolder, mapi_id_t pfid, const gchar *name);
 gboolean		exchange_mapi_connection_remove_folder (ExchangeMapiConnection *conn, mapi_id_t fid);
 gboolean		exchange_mapi_connection_empty_folder (ExchangeMapiConnection *conn, mapi_id_t fid);
@@ -190,7 +199,7 @@ gboolean		exchange_mapi_connection_remove_items (ExchangeMapiConnection *conn, u
 gboolean		exchange_mapi_connection_copy_items (ExchangeMapiConnection *conn, mapi_id_t src_fid, mapi_id_t dest_fid, GSList *mids);
 gboolean		exchange_mapi_connection_move_items (ExchangeMapiConnection *conn, mapi_id_t src_fid, mapi_id_t dest_fid, GSList *mids);
 gboolean 		exchange_mapi_connection_get_folders_list (ExchangeMapiConnection *conn, GSList **mapi_folders);
-gboolean		exchange_mapi_connection_get_pf_folders_list (ExchangeMapiConnection *conn, GSList **mapi_folders, mapi_id_t parent_id);
+gboolean		exchange_mapi_connection_get_pf_folders_list (ExchangeMapiConnection *conn, GSList **mapi_folders);
 GSList *		exchange_mapi_connection_peek_folders_list (ExchangeMapiConnection *conn);
 
 gboolean		exchange_mapi_connection_resolve_named_props (ExchangeMapiConnection *conn, mapi_id_t fid, ResolveNamedIDsData *named_ids_list, guint named_ids_n_elems);
@@ -217,8 +226,13 @@ gboolean		exchange_mapi_connection_events_unsubscribe (ExchangeMapiConnection *c
 
 /* profile functions */
 
+enum {
+	CREATE_PROFILE_FLAG_NONE = 0,
+	CREATE_PROFILE_FLAG_USE_SSL = (1 << 0)
+};
+
 gboolean		exchange_mapi_create_profile (const gchar *username, const gchar *password,
-				       const gchar *domain, const gchar *server,
+				       const gchar *domain, const gchar *server, guint32 flags,
 				       gchar **error_msg, mapi_profile_callback_t cb, gpointer data);
 
 gboolean		exchange_mapi_delete_profile (const gchar *profile);
